@@ -29,6 +29,8 @@ limitations under the License.
 #include "lexer.h"
 #include "unicode.h"
 
+namespace jsonnet::internal {
+
 enum ASTType {
     AST_APPLY,
     AST_APPLY_BRACE,
@@ -584,7 +586,7 @@ struct LiteralNumber : public AST {
 /** Represents JSON strings. */
 struct LiteralString : public AST {
     UString value;
-    enum TokenKind { SINGLE, DOUBLE, BLOCK, VERBATIM_SINGLE, VERBATIM_DOUBLE };
+    enum TokenKind { SINGLE, DOUBLE, BLOCK, VERBATIM_SINGLE, VERBATIM_DOUBLE, RAW_DESUGARED };
     TokenKind tokenKind;
     std::string blockIndent;      // Only contains ' ' and '\t'.
     std::string blockTermIndent;  // Only contains ' ' and '\t'.
@@ -1092,4 +1094,6 @@ auto binary_map = build_binary_map();
 
 }  // namespace
 
+}  // namespace jsonnet::internal
+
 #endif  // JSONNET_AST_H
