@@ -24,6 +24,8 @@ limitations under the License.
 #include "string_utils.h"
 #include "unicode.h"
 
+namespace jsonnet::internal {
+
 static std::string unparse_id(const Identifier *id)
 {
     return encode_utf8(id->name);
@@ -488,6 +490,7 @@ class Unparser {
             o << ast->originalString;
 
         } else if (auto *ast = dynamic_cast<const LiteralString *>(ast_)) {
+            assert(ast->tokenKind != LiteralString::RAW_DESUGARED);
             if (ast->tokenKind == LiteralString::DOUBLE) {
                 o << "\"";
                 o << encode_utf8(ast->value);
@@ -497,7 +500,11 @@ class Unparser {
                 o << encode_utf8(ast->value);
                 o << "'";
             } else if (ast->tokenKind == LiteralString::BLOCK) {
-                o << "|||\n";
+                o << "|||";
+                if (ast->value.back() != U'\n') {
+                    o << "-";
+                }
+                o << "\n";
                 if (ast->value.c_str()[0] != U'\n')
                     o << ast->blockIndent;
                 for (const char32_t *cp = ast->value.c_str(); *cp != U'\0'; ++cp) {
@@ -510,6 +517,9 @@ class Unparser {
                         o << ast->blockIndent;
                     }
                 }
+                if (ast->value.back() != U'\n') {
+                    o << "\n";
+                }
                 o << ast->blockTermIndent << "|||";
             } else if (ast->tokenKind == LiteralString::VERBATIM_DOUBLE) {
                 o << "@\"";
@@ -645,6 +655,7 @@ class EnforceStringStyle : public FmtPass {
     EnforceStringStyle(Allocator &alloc, const FmtOpts &opts) : FmtPass(alloc, opts) {}
     void visit(LiteralString *lit)
     {
+        assert(lit->tokenKind != LiteralString::RAW_DESUGARED);
         if (lit->tokenKind == LiteralString::BLOCK)
             return;
         if (lit->tokenKind == LiteralString::VERBATIM_DOUBLE)
@@ -1879,6 +1890,7 @@ class FixIndentation {
             column += ast->originalString.length();
 
         } else if (auto *ast = dynamic_cast<LiteralString *>(ast_)) {
+            assert(ast->tokenKind != LiteralString::RAW_DESUGARED);
             if (ast->tokenKind == LiteralString::DOUBLE) {
                 column += 2 + ast->value.length();  // Include quotes
             } else if (ast->tokenKind == LiteralString::SINGLE) {
@@ -2318,3 +2330,5 @@ std::string jsonnet_fmt(AST *ast, Fodder &final_fodder, const FmtOpts &opts)
     }
     return ss.str();
 }
+
+}  // namespace jsonnet::internal
