@@ -18,9 +18,12 @@ limitations under the License.
 #define JSONNET_VM_H
 
 #include <libjsonnet.h>
+#include <cstdint>
 
 #include "ast.h"
 
+namespace jsonnet::internal {
+
 /** A single line of a stack trace from a runtime error.
  */
 struct TraceFrame {
@@ -71,7 +74,7 @@ struct VmExt {
  * \param gc_growth_trigger Growth since last garbage collection cycle to trigger a new cycle.
  * \param import_callback A callback to handle imports
  * \param import_callback_ctx Context param for the import callback.
- * \param output_string Whether to expect a string and output it without JSON encoding
+ * \param string_output Whether to expect a string and output it without JSON encoding
  * \throws RuntimeError reports runtime errors in the program.
  * \returns The JSON result in string form.
  */
@@ -95,7 +98,7 @@ std::string jsonnet_vm_execute(Allocator *alloc, const AST *ast,
  * \param gc_growth_trigger Growth since last garbage collection cycle to trigger a new cycle.
  * \param import_callback A callback to handle imports
  * \param import_callback_ctx Context param for the import callback.
- * \param output_string Whether to expect a string and output it without JSON encoding
+ * \param string_output Whether to expect a string and output it without JSON encoding
  * \throws RuntimeError reports runtime errors in the program.
  * \returns A mapping from filename to the JSON strings for that file.
  */
@@ -118,13 +121,34 @@ std::map<std::string, std::string> jsonnet_vm_execute_multi(
  * \param gc_growth_trigger Growth since last garbage collection cycle to trigger a new cycle.
  * \param import_callback A callback to handle imports
  * \param import_callback_ctx Context param for the import callback.
- * \param output_string Whether to expect a string and output it without JSON encoding
+ * \param string_output Whether to expect a string and output it without JSON encoding
  * \throws RuntimeError reports runtime errors in the program.
  * \returns A mapping from filename to the JSON strings for that file.
  */
 std::vector<std::string> jsonnet_vm_execute_stream(
     Allocator *alloc, const AST *ast, const std::map<std::string, VmExt> &ext, unsigned max_stack,
     double gc_min_objects, double gc_growth_trigger, const VmNativeCallbackMap &natives,
-    JsonnetImportCallback *import_callback, void *import_callback_ctx);
+    JsonnetImportCallback *import_callback, void *import_callback_ctx, bool string_output);
+
+/** Safely converts a double to an int64_t, with range and validity checks.
+ *
+ * This function is used primarily for bitwise operations which require integer operands.
+ * It performs two safety checks:
+ * 1. Verifies the value is finite (not NaN or Infinity)
+ * 2. Ensures the value is within the safe integer range [-2^53, 2^53]
+ *
+ * The safe integer range limitation is necessary because IEEE 754 double precision
+ * floating point numbers can only precisely represent integers in the range [-2^53, 2^53].
+ * Beyond this range, precision is lost, which would lead to unpredictable results
+ * in bitwise operations that depend on exact bit patterns.
+ *
+ * \param value The double value to convert
+ * \param loc The location in source code (for error reporting)
+ * \throws StaticError if value is not finite or outside the safe integer range
+ * \returns The value converted to int64_t
+ */
+int64_t safeDoubleToInt64(double value, const LocationRange& loc);
+
+}  // namespace jsonnet::internal
 
 #endif
