@@ -147,7 +147,6 @@ static struct JsonnetJsonValue *cpython_native_callback(
     void *ctx_, const struct JsonnetJsonValue * const *argv, int *succ)
 {
     const struct NativeCtx *ctx = ctx_;
-    int i;
 
     PyEval_RestoreThread(*ctx->py_thread);
 
@@ -156,7 +155,7 @@ static struct JsonnetJsonValue *cpython_native_callback(
 
     // Populate python function args.
     arglist = PyTuple_New(ctx->argc);
-    for (i = 0; i < ctx->argc; ++i) {
+    for (size_t i = 0; i < ctx->argc; ++i) {
         double d;
         const char *param_str = jsonnet_json_extract_string(ctx->vm, argv[i]);
         int param_null = jsonnet_json_extract_null(ctx->vm, argv[i]);
@@ -187,7 +186,7 @@ static struct JsonnetJsonValue *cpython_native_callback(
     }
 
     // Call python function.
-    result = PyEval_CallObject(ctx->callback, arglist);
+    result = PyObject_CallObject(ctx->callback, arglist);
     Py_DECREF(arglist);
 
     if (result == NULL) {
@@ -227,7 +226,7 @@ static int cpython_import_callback(void *ctx_, const char *base, const char *rel
 
     PyEval_RestoreThread(*ctx->py_thread);
     arglist = Py_BuildValue("(s, s)", base, rel);
-    result = PyEval_CallObject(ctx->callback, arglist);
+    result = PyObject_CallObject(ctx->callback, arglist);
     Py_DECREF(arglist);
 
     if (result == NULL) {
@@ -250,22 +249,21 @@ static int cpython_import_callback(void *ctx_, const char *base, const char *rel
 #if PY_MAJOR_VERSION >= 3
         if (!PyUnicode_Check(file_name) || !PyBytes_Check(file_content)) {
 #else
-        if (!PyString_Check(file_name) || !PyString_Check(file_content)) {
+        if (!PyString_Check(file_name) || !PyBytes_Check(file_content)) {
 #endif
-            *buf = jsonnet_str_nonull(ctx->vm, "import_callback did not return (string, bytes)", buflen);
+            *buf = jsonnet_str_nonull(ctx->vm, "import_callback did not return (string, bytes). Since 0.19.0 imports should be returned as bytes instead of as a string.  You may want to call .encode() on your string.", buflen);
             success = 0;
         } else {
-            const char *content_buf;
-            const ssize_t content_len;
+            char *content_buf;
+            Py_ssize_t content_len;
 #if PY_MAJOR_VERSION >= 3
             const char *found_here_cstr = PyUnicode_AsUTF8(file_name);
-            PyBytes_AsStringAndSize(file_content, &content_buf, &content_len);
 #else
             const char *found_here_cstr = PyString_AsString(file_name);
-            PyString_AsStringAndSize(file_content, &content_buf, &content_len);
 #endif
+            PyBytes_AsStringAndSize(file_content, &content_buf, &content_len);
             *found_here = jsonnet_str(ctx->vm, found_here_cstr);
-            *buflen = content_len - 1; // Python always adds a trailing null
+            *buflen = content_len;
             *buf = jsonnet_realloc(ctx->vm, NULL, *buflen);
             memcpy(*buf, content_buf, *buflen);
             success = 1;
@@ -656,7 +654,7 @@ static PyMethodDef module_methods[] = {
 };
 
 #if PY_MAJOR_VERSION >= 3
-static struct PyModuleDef _jsonnet =
+static struct PyModuleDef _module =
 {
     PyModuleDef_HEAD_INIT,
     "_jsonnet",
@@ -667,11 +665,20 @@ static struct PyModuleDef _jsonnet =
 
 PyMODINIT_FUNC PyInit__jsonnet(void)
 {
-    return PyModule_Create(&_jsonnet);
+    PyObject *module = PyModule_Create(&_module);
+    PyObject *version_str = PyUnicode_FromString(LIB_JSONNET_VERSION);
+    if (PyModule_AddObject(module, "version", PyUnicode_FromString(LIB_JSONNET_VERSION)) < 0) {
+      Py_XDECREF(version_str);
+    }
+    return module;
 }
 #else
 PyMODINIT_FUNC init_jsonnet(void)
 {
-    Py_InitModule3("_jsonnet", module_methods, "A Python interface to Jsonnet.");
+    PyObject *module = Py_InitModule3("_jsonnet", module_methods, "A Python interface to Jsonnet.");
+    PyObject *version_str = PyUnicode_FromString(LIB_JSONNET_VERSION);
+    if (PyModule_AddObject(module, "version", PyString_FromString(LIB_JSONNET_VERSION)) < 0) {
+      Py_XDECREF(version_str);
+    }
 }
 #endif
